//
// Copyright (c) 2002-2023 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#include <stdio.h>
#include <stdlib.h>
#include <kdrive_express.h>

#define ERROR_MESSAGE_LEN	(128)	/*!< kdriveExpress Error Messages */
#define MAX_USB_INTERFACES	(10)	/*!< Max usb interfaces */
#define SERIAL_NUMBER_LENGTH (6) 	/*!< The length of a KNX Serial Number */

/*******************************
** Private Functions
********************************/

/*!
	Called when an error occurs
*/
static void error_callback(error_t e, void* user_data);

/*******************************
** Main
********************************/

int main(int argc, char* argv[])
{
	int32_t ap = 0; /*! the Access Port descriptor */
	usb_dev_t iface_items[MAX_USB_INTERFACES]; /*!< holds the usb interface info found by USB enumeration */
	uint32_t iface_items_length = MAX_USB_INTERFACES; /*!< holds the number of items in iface_items */

	/*
		Configure the logging level and console logger
	*/
	kdrive_logger_set_level(KDRIVE_LOGGER_INFORMATION);
	kdrive_logger_console();

	/*
		We register an error callback as a convenience logger function to
		print out the error message when an error occurs.
	*/
	kdrive_register_error_callback(&error_callback, NULL);

	/*
		We create a Access Port descriptor. This descriptor is then used for
		all calls to that specific access port.
	*/
	ap = kdrive_ap_create();

	/*
		We check that we were able to allocate a new descriptor
		This should always happen, unless a bad_alloc exception is internally thrown
		which means the memory couldn't be allocated.
	*/
	if (ap == KDRIVE_INVALID_DESCRIPTOR)
	{
		kdrive_logger(KDRIVE_LOGGER_FATAL, "Unable to create access port. This is a terminal failure");
		return -1;
	}

	kdrive_ap_enum_usb_ex(iface_items, &iface_items_length);
	kdrive_logger_ex(KDRIVE_LOGGER_INFORMATION, "Found %d KNX USB Interfaces", iface_items_length);

	/* If we found at least 1 interface we simply open the first one (i.e. index 0) */
	if ((iface_items_length > 0) && (kdrive_ap_open_usb(ap, 0) == KDRIVE_ERROR_NONE))
	{
		// read serial number via M_Prop
		uint8_t data[SERIAL_NUMBER_LENGTH];
		uint32_t data_length = SERIAL_NUMBER_LENGTH;

		if (kdrive_ap_m_prop_read(ap, 0, 1, 11, 1, 1, data, &data_length) == KDRIVE_ERROR_NONE)
		{
			kdrive_logger_dump(KDRIVE_LOGGER_INFORMATION, "Read Serial Number: ", data, data_length);
		}

		/* close the access port */
		kdrive_ap_close(ap);
	}

	/* releases the access port */
	kdrive_ap_release(ap);

	return 0;
}


/*!
	Called when a kdrive error exception is raised.
	The handling in the error callback is typically
	application specific. And here we simply show
	the error message.
*/
void error_callback(error_t e, void* user_data)
{
	if (e != KDRIVE_TIMEOUT_ERROR)
	{
		static char error_message[ERROR_MESSAGE_LEN];
		kdrive_get_error_message(e, error_message, ERROR_MESSAGE_LEN);
		kdrive_logger_ex(KDRIVE_LOGGER_ERROR, "kdrive error: %s", error_message);
	}
}
